@@ -2,6 +2,7 @@
 /***************************************************************************
  *
  * Copyright (C) 2008 Dan Williams, <dcbw@redhat.com>
+ * Copyright (C) 2008 - 2011 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +34,6 @@
 
 #include <glib.h>
 #include <glib/gi18n-lib.h>
-#include <glade/glade.h>
 
 #include <nm-connection.h>
 #include <nm-setting-vpn.h>
@@ -99,7 +99,7 @@ advanced_dialog_new_hash_from_connection (NMConnection *connection,
 }
 
 static void
-handle_mppe_changed (GtkWidget *check, gboolean is_init, GladeXML *xml)
+handle_mppe_changed (GtkWidget *check, gboolean is_init, GtkBuilder *builder)
 {
 	GtkWidget *widget;
 	gboolean use_mppe;
@@ -109,6 +109,20 @@ handle_mppe_changed (GtkWidget *check, gboolean is_init, GladeXML *xml)
 
 	use_mppe = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
 
+	/* (De)-sensitize MPPE related stuff */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_mppe_security_label"));
+	gtk_widget_set_sensitive (widget, use_mppe);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_mppe_security_combo"));
+	if (!use_mppe)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0); /* default */
+	gtk_widget_set_sensitive (widget, use_mppe);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_stateful_mppe"));
+	if (!use_mppe)
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+	gtk_widget_set_sensitive (widget, use_mppe);
+
 	/* At dialog-setup time, don't touch the auth methods if MPPE is disabled
 	 * since that could overwrite the user's previously chosen auth methods.
 	 * But ensure that at init time if MPPE is on that incompatible auth methods
@@ -120,7 +134,7 @@ handle_mppe_changed (GtkWidget *check, gboolean is_init, GladeXML *xml)
 	/* If MPPE is active, PAP, CHAP, and EAP aren't allowed by the MPPE specs;
 	 * likewise, if MPPE is inactive, sensitize the PAP, CHAP, and EAP checkboxes.
 	 */
-	widget = glade_xml_get_widget (xml, "ppp_auth_methods");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_auth_methods"));
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 	valid = gtk_tree_model_get_iter_first (model, &iter);
 	while (valid) {
@@ -140,25 +154,12 @@ handle_mppe_changed (GtkWidget *check, gboolean is_init, GladeXML *xml)
 
 		valid = gtk_tree_model_iter_next (model, &iter);
 	}
-
-	widget = glade_xml_get_widget (xml, "ppp_mppe_security_label");
-	gtk_widget_set_sensitive (widget, use_mppe);
-
-	widget = glade_xml_get_widget (xml, "ppp_mppe_security_combo");
-	if (!use_mppe)
-		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0); /* default */
-	gtk_widget_set_sensitive (widget, use_mppe);
-
-	widget = glade_xml_get_widget (xml, "ppp_allow_stateful_mppe");
-	if (!use_mppe)
-		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
-	gtk_widget_set_sensitive (widget, use_mppe);
 }
 
 static void
 mppe_toggled_cb (GtkWidget *check, gpointer user_data)
 {
-	handle_mppe_changed (check, FALSE, (GladeXML *) user_data);
+	handle_mppe_changed (check, FALSE, (GtkBuilder *) user_data);
 }
 
 #define SEC_INDEX_DEFAULT   0
@@ -166,7 +167,7 @@ mppe_toggled_cb (GtkWidget *check, gpointer user_data)
 #define SEC_INDEX_MPPE_40   2
 
 static void
-setup_security_combo (GladeXML *xml, GHashTable *hash)
+setup_security_combo (GtkBuilder *builder, GHashTable *hash)
 {
 	GtkWidget *widget;
 	GtkListStore *store;
@@ -174,10 +175,10 @@ setup_security_combo (GladeXML *xml, GHashTable *hash)
 	int active = -1;
 	const char *value;
 
-	g_return_if_fail (xml != NULL);
+	g_return_if_fail (builder != NULL);
 	g_return_if_fail (hash != NULL);
 
-	widget = glade_xml_get_widget (xml, "ppp_mppe_security_combo");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_mppe_security_combo"));
 
 	store = gtk_list_store_new (1, G_TYPE_STRING);
 
@@ -211,7 +212,7 @@ setup_security_combo (GladeXML *xml, GHashTable *hash)
 static void
 check_toggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer user_data)
 {
-	GladeXML *xml = GLADE_XML (user_data);
+	GtkBuilder *builder = (GtkBuilder *) user_data;
 	GtkWidget *widget;
 	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
 	GtkTreeModel *model;
@@ -221,7 +222,7 @@ check_toggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer user_da
 	gboolean mschap_state = TRUE;
 	gboolean mschap2_state = TRUE;
 
-	widget = glade_xml_get_widget (xml, "ppp_auth_methods");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_auth_methods"));
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 
 	gtk_tree_model_get_iter (model, &iter, path);
@@ -254,16 +255,19 @@ check_toggled_cb (GtkCellRendererToggle *cell, gchar *path_str, gpointer user_da
 
 		valid = gtk_tree_model_iter_next (model, &iter);
 	}
-	widget = glade_xml_get_widget (xml, "ppp_use_mppe");
+	/* Make sure MPPE is non-sensitive if MSCHAP and MSCHAPv2 are disabled */
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_use_mppe"));
 	if (!mschap_state && !mschap2_state) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 		gtk_widget_set_sensitive (widget, FALSE);
+		/* Make sure also MPPE security combo and stateful checkbox are non-sensitive */
+		mppe_toggled_cb (widget, builder);
 	} else
 		gtk_widget_set_sensitive (widget, TRUE);
 }
 
 static void
-auth_methods_setup (GladeXML *xml, GHashTable *hash)
+auth_methods_setup (GtkBuilder *builder, GHashTable *hash)
 {
 	GtkWidget *widget;
 	GtkListStore *store;
@@ -359,11 +363,11 @@ auth_methods_setup (GladeXML *xml, GHashTable *hash)
 	                    -1);
 
 	/* Set up the tree view */
-	widget = glade_xml_get_widget (xml, "ppp_auth_methods");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_auth_methods"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget), GTK_TREE_MODEL (store));
 
 	check_renderer = GTK_CELL_RENDERER_TOGGLE (gtk_cell_renderer_toggle_new ());
-	g_signal_connect (check_renderer, "toggled", G_CALLBACK (check_toggled_cb), xml);
+	g_signal_connect (check_renderer, "toggled", G_CALLBACK (check_toggled_cb), builder);
 
 	offset = gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (widget),
 	                                                      -1, "", GTK_CELL_RENDERER (check_renderer),
@@ -386,7 +390,7 @@ auth_methods_setup (GladeXML *xml, GHashTable *hash)
 	gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 
 	/* Make sure MPPE is non-sensitive if MSCHAP and MSCHAPv2 are disabled */
-	widget = glade_xml_get_widget (xml, "ppp_use_mppe");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_use_mppe"));
 	if (!mschap_state && !mschap2_state) {
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 		gtk_widget_set_sensitive (widget, FALSE);
@@ -397,31 +401,39 @@ auth_methods_setup (GladeXML *xml, GHashTable *hash)
 GtkWidget *
 advanced_dialog_new (GHashTable *hash)
 {
-	GladeXML *xml;
+	GtkBuilder *builder;
 	GtkWidget *dialog = NULL;
-	char *glade_file = NULL;
+	char *ui_file = NULL;
 	GtkWidget *widget;
 	const char *value;
 	gboolean mppe = FALSE;
+	GError *error = NULL;
 
 	g_return_val_if_fail (hash != NULL, NULL);
 
-	glade_file = g_strdup_printf ("%s/%s", GLADEDIR, "nm-pptp-dialog.glade");
-	xml = glade_xml_new (glade_file, "pptp-advanced-dialog", GETTEXT_PACKAGE);
-	if (xml == NULL)
-		goto out;
+	ui_file = g_strdup_printf ("%s/%s", UIDIR, "nm-pptp-dialog.ui");
+	builder = gtk_builder_new ();
 
-	dialog = glade_xml_get_widget (xml, "pptp-advanced-dialog");
+	if (!gtk_builder_add_from_file (builder, ui_file, &error)) {
+		g_warning ("Couldn't load builder file: %s",
+		           error ? error->message : "(unknown)");
+		g_clear_error (&error);
+		g_object_unref (G_OBJECT (builder));
+		goto out;
+	}
+	gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
+
+	dialog = GTK_WIDGET (gtk_builder_get_object (builder, "pptp-advanced-dialog"));
 	if (!dialog) {
-		g_object_unref (G_OBJECT (xml));
+		g_object_unref (G_OBJECT (builder));
 		goto out;
 	}
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
-	g_object_set_data_full (G_OBJECT (dialog), "glade-xml",
-	                        xml, (GDestroyNotify) g_object_unref);
+	g_object_set_data_full (G_OBJECT (dialog), "gtkbuilder-xml",
+	                        builder, (GDestroyNotify) g_object_unref);
 
-	setup_security_combo (xml, hash);
+	setup_security_combo (builder, hash);
 
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_REQUIRE_MPPE);
 	if (value && !strcmp (value, "yes"))
@@ -435,34 +447,34 @@ advanced_dialog_new (GHashTable *hash)
 	if (value && !strcmp (value, "yes"))
 		mppe = TRUE;
 
-	widget = glade_xml_get_widget (xml, "ppp_use_mppe");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_use_mppe"));
 	if (mppe)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
-	widget = glade_xml_get_widget (xml, "ppp_allow_stateful_mppe");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_stateful_mppe"));
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_MPPE_STATEFUL);
 	if (value && !strcmp (value, "yes"))
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
-	widget = glade_xml_get_widget (xml, "ppp_allow_bsdcomp");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_bsdcomp"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_NOBSDCOMP);
 	if (value && !strcmp (value, "yes"))
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 
-	widget = glade_xml_get_widget (xml, "ppp_allow_deflate");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_deflate"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_NODEFLATE);
 	if (value && !strcmp (value, "yes"))
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 
-	widget = glade_xml_get_widget (xml, "ppp_usevj");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_usevj"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_NO_VJ_COMP);
 	if (value && !strcmp (value, "yes"))
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 
-	widget = glade_xml_get_widget (xml, "ppp_send_echo_packets");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_send_echo_packets"));
 	value = g_hash_table_lookup (hash, NM_PPTP_KEY_LCP_ECHO_INTERVAL);
 	if (value && strlen (value)) {
 		long int tmp_int;
@@ -473,14 +485,14 @@ advanced_dialog_new (GHashTable *hash)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 	}
 
-	auth_methods_setup (xml, hash);
+	auth_methods_setup (builder, hash);
 
-	widget = glade_xml_get_widget (xml, "ppp_use_mppe");
-	handle_mppe_changed (widget, TRUE, xml);
-	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (mppe_toggled_cb), xml);
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_use_mppe"));
+	handle_mppe_changed (widget, TRUE, builder);
+	g_signal_connect (G_OBJECT (widget), "toggled", G_CALLBACK (mppe_toggled_cb), builder);
 
 out:
-	g_free (glade_file);
+	g_free (ui_file);
 	return dialog;
 }
 
@@ -489,7 +501,7 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 {
 	GHashTable *hash;
 	GtkWidget *widget;
-	GladeXML *xml;
+	GtkBuilder *builder;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	gboolean valid;
@@ -498,15 +510,15 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
 
-	xml = g_object_get_data (G_OBJECT (dialog), "glade-xml");
-	g_return_val_if_fail (xml != NULL, NULL);
+	builder = g_object_get_data (G_OBJECT (dialog), "gtkbuilder-xml");
+	g_return_val_if_fail (builder != NULL, NULL);
 
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	widget = glade_xml_get_widget (xml, "ppp_use_mppe");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_use_mppe"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
 
-		widget = glade_xml_get_widget (xml, "ppp_mppe_security_combo");
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_mppe_security_combo"));
 		switch (gtk_combo_box_get_active (GTK_COMBO_BOX (widget))) {
 		case SEC_INDEX_MPPE_128:
 			g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_REQUIRE_MPPE_128), g_strdup ("yes"));
@@ -519,30 +531,30 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog, GError **error)
 			break;
 		}
 
-		widget = glade_xml_get_widget (xml, "ppp_allow_stateful_mppe");
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_stateful_mppe"));
 		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 			g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_MPPE_STATEFUL), g_strdup ("yes"));
 	}
 
-	widget = glade_xml_get_widget (xml, "ppp_allow_bsdcomp");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_bsdcomp"));
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_NOBSDCOMP), g_strdup ("yes"));
 
-	widget = glade_xml_get_widget (xml, "ppp_allow_deflate");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_allow_deflate"));
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_NODEFLATE), g_strdup ("yes"));
 
-	widget = glade_xml_get_widget (xml, "ppp_usevj");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_usevj"));
 	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
 		g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_NO_VJ_COMP), g_strdup ("yes"));
 
-	widget = glade_xml_get_widget (xml, "ppp_send_echo_packets");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_send_echo_packets"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
 		g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_LCP_ECHO_FAILURE), g_strdup_printf ("%d", 5));
 		g_hash_table_insert (hash, g_strdup (NM_PPTP_KEY_LCP_ECHO_INTERVAL), g_strdup_printf ("%d", 30));
 	}
 
-	widget = glade_xml_get_widget (xml, "ppp_auth_methods");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "ppp_auth_methods"));
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 	valid = gtk_tree_model_get_iter_first (model, &iter);
 	while (valid) {
